package com.guams.review.service;

import com.guams.review.exception.NotFoundException;
import com.guams.review.exception.ForbiddenExecption;
import com.guams.review.model.AuthorRepository;
import com.guams.review.model.PostRepository;
import com.guams.review.model.dao.Author;
import com.guams.review.model.dao.Post;
import com.guams.review.service.mapper.Mapper;
import com.guams.review.service.mapper.ReturnableAuthor;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@RequiredArgsConstructor
public class AuthorService implements UserDetailsService
{
    private final Mapper mapper;
    private final AuthorRepository authorRepository;
    private final PostRepository postRepository;

    public List<ReturnableAuthor> list() {
        return authorRepository.findAll().stream()
                .map(mapper::mapAuthor)
                .toList();
    }

    public List<Post> listPublicationOfAuthor(UUID authorId) {
        return postRepository.findPostsOfAuthor(authorId);
    }

    public Optional<Author> findById(UUID id) {
        return authorRepository.findById(id);
    }

    public ReturnableAuthor insert(Author author) {
        return mapper.mapAuthor(authorRepository.save(author));
    }

    @Transactional
    public void insertPublications(UUID authorId, Long postId) {
            if (postRepository.findById(postId).isPresent()) {
                authorRepository.insertPublication(authorId, postId);
        }
    }

    public Author verifyIfUserIsAuthorized(Authentication authentication, UUID id) {
        if (authentication == null || !authentication.isAuthenticated()) {
            throw new ForbiddenExecption("You have to login first");
        }
        Author author = findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        String username = authentication.getName();
        Author authorAuthenticated = authorRepository.findByName(username).orElseThrow(() -> new NotFoundException("Author not found"));
        if (authorAuthenticated.getId().compareTo(author.getId()) != 0 && !authorAuthenticated.getRole().equals("ADMIN")) {
            throw new ForbiddenExecption("Specified Author is not authorized to do that");
        }
        return author;
    }

    public Optional<Author> findByName(String name) {
        return authorRepository.findByName(name);
    }

    public void delete(Author author) {
        authorRepository.delete(author);
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        Author author = authorRepository.findByName(username).orElseThrow(() -> new UsernameNotFoundException("Author not found"));
        return new User(author.getName(), author.getPassword(), Collections.singletonList(new SimpleGrantedAuthority(author.getRole())));
    }
}
